#pragma once

#include <limits>
#include <vector>
#include <atomic>
#include "../pdv_lib/pdv_lib.hpp"
#include "./3prefixsum.h"

template<typename T>
void countsort_seq(std::vector<T>& data) {
    // find the lowest and highest values in the input
    auto min = std::numeric_limits<T>::max();
    auto max = std::numeric_limits<T>::min();
    for (auto& n : data) {
        if (n < min) min = n;
        if (n > max) max = n;
    }

    // count each value in the vector in the min/max range
    auto range = max - min + 1;
    auto counts = std::vector<size_t>(range);
    for (auto& n : data) {
        counts[n - min]++;
    }

    // reconstruct the vector content from the counts
    auto it = data.begin();
    for (size_t i = 0; i < counts.size(); i++) {
        // set `counts[i]` values to `min + i`, return iterator to the end of the range
        it = std::fill_n(it, counts[i], min + i);
    }
}

template<typename T>
void countsort_par(std::vector<T>& data) {
    // Naimplementujte paralelni verzi counting-sortu. Zamyslete se nad tim,
    // jak muzete paralelizovat:
    //   1) Nalezeni rozsahu hodnot v poli
    //   2) Napocitani "histogramu"
    //   3) "Vypsani" histogramu do pole 'data'
    // Inspirovat se muzete sekvencni implementaci
    throw pdv::not_implemented();
}
